% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This code generates Figure C.1 in the Online Appendix
% (it is called within the file main_run.m)
% =========================================================================


get_setup

% define figure properties
T_plot = 150;

addpath Simulations

 load pf_121; load pf_521; 



%%

opts.Colors     = get(groot,'defaultAxesColorOrder');
opts.saveFolder = 'Figures/';
opts.width      = 17;
opts.height     = 20;
opts.fontType   = 'Times New Roman';
opts.fontSize   = 9;
set(0,'DefaultFigureRenderer','painters')
set(0,'DefaultLineLineWidth',1.5)
set(0,'DefaultLineMarkerSize',4)

fig = figure; clf
subplot(2,2,1);hold on;grid on;box on;
plot(100*pf_121.A(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_521.A(1:T_plot),'r--','LineWidth',1.5);
title('Infected (Asymptomatic)');ylabel('% of population');xlabel('Weeks')

subplot(2,2,2);hold on;grid on;box on;
plot(100*pf_121.D(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_521.D(1:T_plot),'r--','LineWidth',1.5);
title('Deaths');xlabel('Weeks');ylabel('% of population')

subplot(2,2,3);hold on;grid on;box on;
plot(100*(pf_121.Cons(1:T_plot)-P.C)/P.C,'b','LineWidth',1.5);
plot(100*(pf_521.Cons(1:T_plot)-P.C)/P.C,'r--','LineWidth',1.5);
title('Aggregate Consumption');ylabel('% dev. from initial ss');xlabel('Weeks')

subplot(2,2,4);hold on;grid on;box on;
plot(100*pf_121.pic_T(1:T_plot),'b','LineWidth',1.5);
plot(100*pf_521.pic_T(1:T_plot),'r--','LineWidth',1.5);
ylim([0 45])
title('Test Probability Newly Infected \pi_t^T');ylabel('%');xlabel('Weeks')


l1 = legend('Contact Tracing','Contact Tracing + Random Testing','Location','South','Orientation','horizontal');
l1.Position(2) = 0.015;l1.Position(1) = 0.21;

% scaling
fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height./2;
set(fig.Children,'FontName','Times','FontSize',9);

print([opts.saveFolder 'FigureC1_MR_JEEA'], '-dpng', '-r600')
print([opts.saveFolder 'FigureC1_MR_JEEA'], '-depsc2', '-r600')
