% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This code generates Figure G.1 in the Online Appendix
% (it is called within the file main_run.m)
% =========================================================================

% Some additional calculations

get_setup
P.tol = 0.1;

P.start_nguess = 1;
P.Choice_Ups = 2;
P.trace = 1;
P.tau = 0.05;
P.start_eps = 1;
[pf] = eqm_Exposure(P,T);
pf_0 = pf;

P.start_ns = pf.n_s;  % Labor of susceptible
P.start_ni = pf.n_i;  % Labor of infected     
P.start_nr = pf.n_r;  % Labor of recovered 
P.start_nc = pf.n_c;  % Labor of checked
P.start_na = pf.n_a;  % Labor of asymptotic 
P.start_nq = pf.n_q;  % Labor of quarantined agents
P.start_nguess = 2;
    
P.Choice_Ups = 4;
P.Upsilon_adjusted = [pf.E_tt;pf.E_tt(end)]+0.002;

t_sel = 42; 
P.Upsilon_adjusted = pf.Upsilon(t_sel:end);
P.start_ns = pf.n_s(t_sel:end);  % Labor of susceptible
P.start_ni = pf.n_i(t_sel:end);  % Labor of infected     
P.start_nr = pf.n_r(t_sel:end);  % Labor of recovered 
P.start_nc = pf.n_c(t_sel:end);  % Labor of checked
P.start_na = pf.n_a(t_sel:end);  % Labor of asymptotic 
P.start_nq = pf.n_q(t_sel:end);  % Labor of quarantined agents

 % How many periods after
P.Pop_start=  pf_0.Pop(t_sel,1);
P.A_start= pf_0.A(t_sel,1); 
P.I_start=  pf_0.I(t_sel,1); 
P.S_start= pf_0.S(t_sel,1);
P.RA_start= pf_0.RA(t_sel,1);
P.RI_start= pf_0.RI(t_sel,1); 
P.D_start=  pf_0.D(t_sel,1);
P.C_start= pf_0.C(t_sel,1); 
P.QS_start= pf_0.QS(t_sel,1);
P.QA_start= pf_0.QA(t_sel,1);
P.QU_start= pf_0.QU(t_sel,1); 
    
P.Choice_Ups = 4;
P.start_eps = 0;
P.tau = 0.045;
[pf] = eqm_Exposure(P,T-t_sel+1);
pf_1 = pf;

P.Choice_Ups = 4;
P.start_eps = 0;
P.tau = 0.025;
[pf] = eqm_Exposure(P,T-t_sel+1);
pf_2 = pf;

%%
opts.Colors     = get(groot,'defaultAxesColorOrder');
opts.saveFolder = 'Figures/';
opts.width      = 17;
opts.height     = 10;
opts.fontType   = 'Times New Roman';
opts.fontSize   = 9;
set(0,'DefaultFigureRenderer','painters')
set(0,'DefaultLineLineWidth',1.5)
set(0,'DefaultLineMarkerSize',4)


t_start = 35;
t_plot = 45-t_sel+1;

fig = figure; clf

subplot(2,2,1);hold on;grid on;box on;
plot(t_start:t_sel+t_plot-1,100*[pf_0.T(t_start:t_sel-1); pf_1.T(1:t_plot)],'r--')
plot(t_start:t_sel+t_plot-1,100*[pf_0.T(t_start:t_sel-1); pf_2.T(1:t_plot)],'k-.')
plot(t_start:t_sel+t_plot-1,100*pf_0.T(t_start:t_sel+t_plot-1),'b')
plot([41 41],[0 3],'k:')
xlim([t_start,t_sel+t_plot-1])
ylabel('% of population');title('Newly Infected')

subplot(2,2,2);hold on;grid on;box on;
plot(t_start:t_sel+t_plot-1,100*([pf_0.Cons(t_start:t_sel-1); pf_1.Cons(1:t_plot)]-P.C)/P.C,'r--')
plot(t_start:t_sel+t_plot-1,100*([pf_0.Cons(t_start:t_sel-1); pf_2.Cons(1:t_plot)]-P.C)/P.C,'k-.')
plot(t_start:t_sel+t_plot-1,100*(pf_0.Cons(t_start:t_sel+t_plot-1)-P.C)/P.C,'b');
plot([41 41],[-50 0],'k:')
ylim([-50 0])
xlim([t_start,t_sel+t_plot-1])
ylabel('% dev. from initial ss');title('Aggregate Consumption')

subplot(2,2,3);hold on;grid on;box on;
plot(t_start:t_sel+t_plot-1,100*[pf_0.pic_T(t_start:t_sel-1); pf_1.pic_T(1:t_plot)],'r--')
plot(t_start:t_sel+t_plot-1,100*[pf_0.pic_T(t_start:t_sel-1); pf_2.pic_T(1:t_plot)],'k-.')
plot(t_start:t_sel+t_plot-1,100*pf_0.pic_T(t_start:t_sel+t_plot-1),'b')
plot([41 41],[100*0.05 100*0.2],'k:')
xlim([t_start,t_sel+t_plot-1])
ylim([100*0.05 100*0.2])
title('Test Probability Newly Infected');ylabel('%')

subplot(2,2,4);hold on;grid on;box on;
ll2 = plot(t_start:t_sel+t_plot-1,100*[pf_0.E_tt(t_start:t_sel-1); pf_1.E_tt(1:t_plot)],'r--'); 
ll3 = plot(t_start:t_sel+t_plot-1,100*[pf_0.E_tt(t_start:t_sel-1); pf_2.E_tt(1:t_plot)],'k-.');
ll1 = plot(t_start:t_sel+t_plot-1,100*pf_0.E_tt(t_start:t_sel+t_plot-1),'b');
plot([41 41],[100*0.05 100*0.25],'k:')
ylim([100*0.05 100*0.25])
xlim([t_start,t_sel+t_plot-1])
title('Traced Contacts');ylabel('% of population')


l1 = legend([ll1,ll2,ll3],'Baseline','Mask Mandate: Less Stringent','Mask Mandate: More Stringent','Orientation','Horizontal');
l1.Position(2) = 0.0075;l1.Position(1) = 0.0825;
axis tight

fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;

print([opts.saveFolder 'FigureG1_MR_JEEA'], '-dpng', '-r600')
print([opts.saveFolder 'FigureG1_MR_JEEA'], '-depsc2', '-r600')