% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This code generates Figure I.2 in the Online Appendix
% (it is called within the file main_run.m)
% =========================================================================

get_setup

opts.Colors     = get(groot,'defaultAxesColorOrder');
opts.saveFolder = 'Figures/';
opts.width      = 17;
opts.height     = 8;
opts.fontType   = 'Times New Roman';
opts.fontSize   = 9;
set(0,'DefaultFigureRenderer','painters')
set(0,'DefaultLineLineWidth',1.5)
set(0,'DefaultLineMarkerSize',4)

load pf_111;pf = pf_111;

tau_tilde = (1 - (1-P.tau).^P.grid)'; 
t = 20;
[f_k] = pmf_contacts_withA(pf.c_s(t,1),pf.n_s(t,1),t,P,pf);
f_k_20 = f_k;  f_SA_20 = f_k.*tau_tilde/pf.tau_exact(t,1);  % Newly Infected

t = 40;
[f_k] = pmf_contacts_withA(pf.c_s(t,1),pf.n_s(t,1),t,P,pf);
f_k_40 = f_k;
f_SA_40 = f_k.*tau_tilde/pf.tau_exact(t,1);  % Newly Infected
        
fig = figure; clf
subplot(1,2,1);box on;
b1 = bar(P.grid(1:7),[f_k_20(1:7),f_k_40(1:7)],'grouped');
b1(1).FaceColor = [0 0.4470 0.7410];
b1(1).EdgeColor='none';b1(2).FaceColor='none';b1(2).EdgeColor='k';
xlabel('Number of meetings with asymptomatic (k)')
title('Susceptibles and Untested Asymptomatic')

subplot(1,2,2);box on;
b1 = bar(P.grid(1:7),[f_SA_20(1:7),f_SA_40(1:7)],'grouped');
b1(1).FaceColor = [0 0.4470 0.7410];
b1(1).EdgeColor='none';
b1(2).FaceColor='none';
b1(2).EdgeColor='k';
legend('t=20','t=40')
xlabel('Number of meetings with asymptomatic (k)')
title('Newly Infected')


fig.Units               = 'centimeters';
fig.Position(3)         = opts.width;
fig.Position(4)         = opts.height;
set(fig.Children,'FontName','Times','FontSize',9);

print([opts.saveFolder 'FigureI2_MR_JEEA'], '-dpng', '-r600')
print([opts.saveFolder 'FigureI2_MR_JEEA'], '-depsc2', '-r600')