% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This function solves the macro-SIR model numerically (see Online Appendix D)
% =========================================================================

function [pf] = eqm_Exposure(P,T)  %%  SIRAC model



%call_SIRAC;
%pf.R_0_Sir = (pf.T_Sir(:)./pf.A_Sir(:))./(P.pir+P.pii); %./(P.pir+P.pid+P.pii)
%disp(['Sirac R_0 ' num2str(pf.R_0_Sir(1))])

%% Create Policy Functions

% Exogenous shock - government function
pf.lambda_c = zeros(T,1);  % 
% Other policy functions 
pf.c_r = zeros(T,1); % Consumption of recovered
pf.lambdab_r =  zeros(T,1); % Marginal utility of recovered
pf.u_r = zeros(T,1); % Utility of recovered in period t
pf.Util_r = zeros(T+1,1);  % Total Utility
pf.c_i = zeros(T,1); % Consumption of infected
pf.lambdab_i =  zeros(T,1); % Marginal utility of infected
pf.u_i = zeros(T,1); % Utility of infected in period t
pf.Util_i = zeros(T+1,1);  % Total Utility
pf.c_a = zeros(T,1); % Consumption of asymptotic infected
pf.u_a = zeros(T,1); % Utility of asympotic infected in period t
pf.Util_a = zeros(T+1,1);  % Total Utility of asymptotic infected
pf.c_c = zeros(T,1); % Consumption of checked infected
pf.lambdab_c =  zeros(T,1); % Marginal utility of checked infected 
pf.u_c = zeros(T,1); % Utility of checked infected in period t
pf.Util_c = zeros(T+1,1);  % Total Utility of asymptotic infected
pf.c_s = zeros(T,1); % Consumption of susceptible
pf.lambdab_s =  zeros(T,1); % Marginal utility of susceptible
pf.lambdatau_s = zeros(T,1); % Another marginal utility of susceptible
pf.u_s = zeros(T,1); % Utility of susceptible in period t
pf.Util_s = zeros(T+1,1);  % Total Utility
pf.Lambda = zeros(T,1);  % Policy Function
pf.Util_ur = zeros(T+1,1);  % Unobsered Recovered Utility
pf.lambdab_q =  zeros(T,1); % Marginal utility of quarantined
pf.c_q = zeros(T,1); % Consumption of quarantined symptomatic
pf.u_q = zeros(T,1); % Utility of quarantined symptomatic in period t
pf.Util_qs = zeros(T+1,1);  % Total Utility of quarantined symptotic
pf.Util_qa = zeros(T+1,1);  % Total Utility of quarantined asymptotic
pf.Util_qa = zeros(T+1,1);  % Total Utility of quarantined unobserved recovered
% State variables for population
pf.T   = zeros(T,1);  % Case of New Transmission in t+1
pf.tau = zeros(T,1);
pf.pid_t = zeros(T,1);
pf.Pop = ones(T,1);  % Total Population
pf.S   = ones(T,1);  % Susceptible
pf.I   = zeros(T,1);  % Infected
pf.R   = zeros(T,1);  % Recovered
pf.RA   = zeros(T,1);  % Recovered
pf.RI   = zeros(T,1);  % Recovered
pf.C   = zeros(T,1);  % Checked
pf.A   = zeros(T,1);  % Asymptotic
pf.D   = zeros(T,1);  % Deceased

pf.Q  = zeros(T,1);     % All Quarantined
pf.QS =  zeros(T,1);    % Quarantined Symptomatic
pf.QA =  zeros(T,1);    % Quarantined Asymptomatic
pf.QU =  zeros(T,1);    % Quarantined Unobserved Recovered

pf.pic = zeros(T,1);
pf.Upsilon = zeros(T,1); % Amount of Tests
pf.pic_A = zeros(T,1); % Probability of getting tested
pf.pic_T = zeros(T,1); % Probability of getting tested
pf.pic_A_tt = zeros(T,1); % Probability of getting tested
pf.pic_T_tt = zeros(T,1); % Probability of getting tested
pf.pic_A_ttl = zeros(T,1); % Probability of getting tested
pf.pic_T_ttl = zeros(T,1); % Probability of getting tested
pf.fk        = zeros(T,length(P.grid));

pf.piq_ST = zeros(T,1); % Probability of getting quarantined susceptibles wo newly infected
pf.piq_T  = zeros(T,1); % Probability of getting quarantined newly infected
pf.piq_A  = zeros(T,1); % Probability of getting quarantined asymptomatic
pf.piq_U  = zeros(T,1); % Probability of getting quarantined unobsered recovered
% Distribution of contacts
pf.g_A = zeros(P.k_s_max+1,T);
pf.g_ST = zeros(P.k_s_max+1,T);
pf.g_RA = zeros(P.k_s_max+1,T);

% Labor policy functions for the different periods

if P.start_nguess == 1

    pf.n_s = P.N*ones(T,1);  % Labor of susceptible
    pf.n_i = P.N*ones(T,1);  % Labor of infected     
    pf.n_r = P.N*ones(T,1);  % Labor of recovered 
    pf.n_c = P.N*ones(T,1);  % Labor of checked
    pf.n_a = P.N*ones(T,1);  % Labor of asymptotic 
    pf.n_q = P.N*ones(T,1);  % Labor of quarantined agents

else
    pf.n_s = P.start_ns;  % Labor of susceptible
    pf.n_i = P.start_ni;  % Labor of infected     
    pf.n_r = P.start_nr;  % Labor of recovered 
    pf.n_c = P.start_nc;  % Labor of checked
    pf.n_a = P.start_na;  % Labor of asymptotic 
    pf.n_q = P.start_nq;  % Labor of quarantined agents
end

% Store updated policy functions
pf_up_n_r = zeros(T,1); 
pf_up_n_i = zeros(T,1);
pf_up_n_s = zeros(T,1);
pf_up_n_c = zeros(T,1);
pf_up_n_q = zeros(T,1);

% Calculuate 
pf.Util_r(T+1,1) = P.Util;  % This is the future
pf.Util_i(T+1,1) = P.Util_i;  %P.Util;
pf.Util_s(T+1,1) = P.Util;
pf.Util_ur(T+1,1) = P.Util;
pf.Util_c(T+1,1) = P.Util_c;
pf.Util_a(T+1,1) = P.Util_a;

pf.Util_qs(T+1,1) = P.Util;
pf.Util_qa(T+1,1) = P.Util_a;
pf.Util_qu(T+1,1) = P.Util;

%%
if P.Choice_lock == 1
elseif P.Choice_lock == 2
    pf.lambda_c(P.lock_start:P.lock_end) = P.lockdown; 0.5;
elseif P.Choice_lock == 3

elseif P.Choice_lock == 4
     pf.lambda_c(5) = P.lockdown;
elseif P.Choice_lock == 5
     pf.lambda_c(P.lock_start:P.lock_end) = P.lockdown; 0.5;
     pf.lambda_c(35) = -0.25;
 elseif P.Choice_lock == 6
    pf.lambda_c(P.lock_start_6:P.lock_end_6) = P.lockdown_6; 0.5;
elseif P.Choice_lock == 7  % Optimal Policy
    pf.lambda_c(1:numel(P.muc_sol_in))= P.muc_sol_in;
end


% Use this to find good start point
repeat_length = 1;
if P.trace==1 &&  P.Choice_Ups == 2 && P.Choice_lock == 7 && sum(pf.lambda_c>0)==150
    repeat_length = 2;
end


for repeat = 1:repeat_length 

    if P.trace==1 &&  P.Choice_Ups == 2 && P.Choice_lock == 7 && sum(pf.lambda_c>0)==150
        P.Choice_Ups =1;
        tol_old = P.tol;
        P.tol = 0.25;
    elseif repeat == 2
        P.Choice_Ups = 2;
        P.tol = tol_old;
    end      
        
        
   
    %% Main Model

    converged = 0;  % Inititaluz
    j = 1;  % Number of Iteration
    % Calendat week 11 (March 16) and of september week 40   
    if P.Choice_Ups == 1  %Flatrate of 10% Tests
        pf.Upsilon(1:T) = P.Ups_flatrate; 
    elseif P.Choice_Ups == 2  % Limited Tests
        pf.Upsilon(1:29) = linspace(P.Ups_start_w11,P.Ups_end_w40,29);  
        P.step_s = pf.Upsilon(2)-pf.Upsilon(1);  
        pf.Upsilon(30:52) = P.Ups_end_w40+P.step_s :P.step_s:P.Ups_end_w40 +23*P.step_s;
        pf.Upsilon(53:T) =   pf.Upsilon(52);
    elseif P.Choice_Ups == 3
    elseif P.Choice_Ups == 4  % Read a specific Upsilon in
        pf.Upsilon = P.Upsilon_adjusted;    
    end


    dist_total = 0;
    
    
    
    while converged == 0
 
        if P.Choice_lock == 3
            alphas  = P.alphas_in;
            pf.lambda_c(:) = max(0,alphas(1) + alphas(2)*pf.D(:) + alphas(3)*pf.D(:).^2 + alphas(4)*pf.D(:).^3);
        end

   

        % Aggregate Labor and Consumption
        pf.N    = pf.S.*pf.n_s+ P.phi*pf.I.*pf.n_i + pf.RA.*pf.n_s + pf.RI.*pf.n_r + pf.A.*pf.n_s + P.phi_c*pf.C.*pf.n_c + P.phi_q*pf.Q.*pf.n_q; 
        pf.Cons = pf.S.*pf.c_s+       pf.I.*pf.c_i + pf.RA.*pf.c_s + pf.RI.*pf.c_r + pf.A.*pf.c_s +         pf.C.*pf.c_c + pf.Q.*pf.c_q;

        % Wage  
        pf.w = ones(T,1)*P.A;

        % Compute a sequence of remaining unknown variables using backward
        % iteration

        for t = T:-1:1


            % Block for the recovered
            % Marginal utility of recovered
            pf.lambdab_r(t,1) = P.theta*pf.n_r(t,1).^(1/P.eta-1)./(pf.w(t,1));
            % Consumption of recovered
            pf.c_r(t,1) = ((1+pf.lambda_c(t,1)).*pf.lambdab_r(t,1) ).^-1;
            % Utility of recovered in period t
            pf.u_r(t,1) = log(pf.c_r(t,1)) - P.theta/(1/P.eta)*(pf.n_r(t,1)).^(1/P.eta);
            % Utility function
            pf.Util_r(t,1) = pf.u_r(t,1) + P.beta.* pf.Util_r(t+1,1);
            % Lambda via Budget Constraint of Recovered
            pf.Lambda(t,1) = (1+pf.lambda_c(t,1))*pf.c_r(t,1) - pf.w(t,1)*pf.n_r(t,1);

            % Block for the infected
            % Marginal uility of infected 
            pf.lambdab_i(t,1) = P.theta/(pf.w(t,1)*P.phi).* pf.n_i(t,1).^(1/P.eta-1);
            % Consumption of infected 
            pf.c_i(t,1) = ((1+P.alpha_c*pf.lambda_c(t,1)+P.mu_cQ).*pf.lambdab_i(t,1) ).^-1;  
            % Utility of recovered in period t
            pf.u_i(t,1) = log(pf.c_i(t,1)) - P.theta/(1/P.eta)*(pf.n_i(t,1)).^(1/P.eta); 

            % Block for the checked
            % Marginal uility of checked
            pf.lambdab_c(t,1) = P.theta/(pf.w(t,1)*P.phi_c).* pf.n_c(t,1).^(1/P.eta-1);
            % Consumption of checked
            pf.c_c(t,1) = ((1+P.alpha_c*pf.lambda_c(t,1)+P.mu_cQ).*pf.lambdab_c(t,1) ).^-1;  
            % Utility of checked in period t
            pf.u_c(t,1) = log(pf.c_c(t,1)) - P.theta/(1/P.eta)*(pf.n_c(t,1)).^(1/P.eta); 

            % Block for the quarantined
            % Marginal uility of quarantined
            pf.lambdab_q(t,1) = P.theta/(pf.w(t,1)*P.phi_q).* pf.n_q(t,1).^(1/P.eta-1);
            % Consumption of checked
            pf.c_q(t,1) = ((1+P.alpha_c*pf.lambda_c(t,1)+P.mu_cQ).*pf.lambdab_q(t,1) ).^-1;  
            % Utility of checked in period t
            pf.u_q(t,1) = log(pf.c_q(t,1)) - P.theta/(1/P.eta)*(pf.n_q(t,1)).^(1/P.eta); 


            % Block for the susceptible
            % Consumption of susceptible (via budget constraint)
            pf.c_s(t,1) = (1/(1+pf.lambda_c(t,1)))*(pf.w(t,1)*pf.n_s(t,1) + pf.Lambda(t,1) );      
            % Utility of recovered in period t
            pf.u_s(t,1) = log(pf.c_s(t,1)) - P.theta/(1/P.eta)*(pf.n_s(t,1)).^(1/P.eta);   
            % Block for the asymptotic        

            pf.c_a(t,1) = pf.c_s(t,1);
            pf.n_a(t,1) = pf.n_s(t,1);
            pf.u_a(t,1) = pf.u_s(t,1);

        end

        % Use forward iteration:

        if P.start_eps == 1  %No starting point given 
            pf.Pop(1,1) = 1;
            pf.A(1,1) = P.eps_A*P.epsilon;
            pf.I(1,1) = P.eps_I*P.epsilon;
            pf.S(1,1) = pf.Pop(1,1)-pf.A(1,1)-pf.I(1,1);
            pf.RA(1,1) = P.eps_RA*P.epsilon;
            pf.RI(1,1) = P.eps_RI*P.epsilon;
            pf.R(1,1) = pf.RA(1,1) + pf.RI(1,1) ;
            pf.D(1,1) = 0;
            pf.C(1,1) = 0;

            % Quarantined agents
            pf.QS(1,1) = 0;
            pf.QA(1,1) = 0.0;
            pf.QU(1,1) = 0.0;
            pf.Q(1,1) = pf.QS(1,1) + pf.QA(1,1) + pf.QU(1,1);

        else
            pf.Pop(1,1) = P.Pop_start;
            pf.A(1,1) = P.A_start;
            pf.I(1,1) =  P.I_start;
            pf.S(1,1) = P.S_start;
            pf.RA(1,1) = P.RA_start;
            pf.RI(1,1) = P.RI_start;
            pf.D(1,1) = P.D_start;
            pf.C(1,1) = P.C_start;
            pf.QS(1,1) = P.QS_start;
            pf.QA(1,1) = P.QA_start;
            pf.QU(1,1) = P.QU_start;
            pf.R(1,1) = pf.RA(1,1) + pf.RI(1,1) ;
            pf.Q(1,1) = pf.QS(1,1) + pf.QA(1,1) + pf.QU(1,1);
        end



        for t = 1:T     % Starting point given


            % Death Rate
            pf.pid_t(t,1) = P.pid + P.kappa*pf.I(t,1)^2;

            %%%%%%%%%%% Calculate T %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
            % Aggregate Consumption for Meeting 
            pf.Cons_Meet(t,1) = pf.S(t,1).*pf.c_s(t,1) + pf.RA(t,1).*pf.c_s(t,1) + pf.RI(t,1).*pf.c_r(t,1) + pf.A(t,1).*pf.c_s(t,1)  ;

            % Aggregate Labor  for Meeting 
            pf.N_Meet(t,1) = pf.S(t,1).*pf.n_s(t,1) + pf.RA(t,1).*pf.n_s(t,1)  + pf.RI(t,1).*pf.n_r(t,1)  + pf.A(t,1).*pf.n_s(t,1) ;

            % Population 
            pf.Pop_Meet(t,1) = pf.Pop(t,1) - pf.C(t,1) -pf.I(t,1) - pf.Q(t,1);

            % Some aggregate values and for the asymptomatics / necessary for
            % newly infected
            C_A = pf.c_s(t,1)*pf.A(t,1);
            N_A = pf.n_s(t,1)*pf.A(t,1);
            C_Pop = pf.Cons_Meet(t,1); %*pf.Pop_Sir(t,1); % Note that the calculation already includes the reduced population weight
            N_Pop = pf.N_Meet(t,1); %*pf.Pop_Sir(t,1);



            % Probability mass function of contacts from susceptibles with asymptomatic 
            [f_k] = pmf_contacts_withA(pf.c_s(t,1),pf.n_s(t,1),t,P,pf);
            pf.fk(t,:) = f_k;
            % Probability to get infected (based on Bernoulli trials)
            tau_tilde = (1 - (1-P.tau).^P.grid)';

            % Distribution of newly infected 
            pf.tau_exact(t,1) = sum(f_k.*tau_tilde);


            % Law of Motion for Newly Infected(Linearized)
            k_bar = (P.varphi_c*pf.c_s(t,1))*C_A/C_Pop + (P.varphi_n*pf.n_s(t,1))*N_A/N_Pop + (P.varphi)*pf.A(t,1)/pf.Pop_Meet(t,1);
            Xi = -log(1-P.tau)*(1-P.tau)^k_bar; 
            T_new = Xi*k_bar*pf.S(t,1); 

            % Define pi_1,pi_2,pi_3 (Note that they are time-varying
            pf.pi_s1(t,1) = Xi*P.varphi_c;
            pf.pi_s2(t,1) = Xi*P.varphi_n; 
            pf.pi_s3(t,1) = Xi*P.varphi;

            % New cases 
            pf.T(t,1) = T_new;

            % Relative Infection Probability for susceptible               
            pf.tau(t,1) = pf.T(t,1)./pf.S(t,1);


            if t < T
            %%%%%%%%%% Tracing and Testing %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

            % No Tracing and Testing


            if P.trace >= 1  && P.trace <10 % Scenario 1

                % Probability of being caught expressed as Bernoulli trial
                mu_tilde = (1 - (1-P.pii).^P.grid)';

                %%%% Susceptible %%%%
                % Probability of a susceptible have k contacts conditional on
                % the subject to become infected
                f_SA_T = f_k.*tau_tilde/pf.tau_exact(t,1);  % Newly Infected
                f_SA_ST = f_k.*(1-tau_tilde)/(1-pf.tau_exact(t,1)); % Remaining Susceptibles

                % Average Probability of being Traced            
                pi_C_S_tt = f_k'*mu_tilde;  % Susceptibles at beginning of period, Asymptomatics and Untested Recovered
                pi_C_T_tt = f_SA_T'*mu_tilde; % Newly Infected
                pi_C_ST_tt = f_SA_ST'*mu_tilde; % Susceptibles without Newly Infected
                pi_C_A_tt = pi_C_S_tt;    % Asymptomatic 
                pi_C_RA_tt = pi_C_S_tt;   % Undetected Recovered

                % Contacts
                pf.E_tt(t,1) = pi_C_S_tt*pf.S(t,1) + pi_C_A_tt*(1-P.pii)*pf.A(t,1) + pi_C_RA_tt*pf.RA(t,1);
                % Probability of getting tested conditional on being traced
                pf.pi_bar_tt(t,1) = min(pf.Upsilon(t,1)./pf.E_tt(t,1),1);

                %
                pf.pic_A_tt(t,1) = pi_C_A_tt*pf.pi_bar_tt(t,1)*(1-P.pi_F);
                pf.pic_T_tt(t,1) = pi_C_T_tt*pf.pi_bar_tt(t,1)*(1-P.pi_F);

                % Aggregate 
                pf.pic_A(t,1)  = pf.pic_A_tt(t,1);
                pf.pic_T(t,1)  = pf.pic_T_tt(t,1);


                if P.trace == 5 % Combine basic tracing with random testing
                    % Note that basic tracing has been completed by now. We now
                    % move to random testing with the remaining tests

                    % All possible traceable agents 
                    All_traceable = pf.S(t,1) + (1-P.pii)*pf.A(t,1) + pf.RA(t,1);
                    Remainder_traceable = All_traceable -  pf.E_tt(t,1) ; % Adjust for agents that have already been traced
                    Remainder_tests = max(pf.Upsilon(t,1)-pf.E_tt(t,1),0);  % 
                     % Probability of getting tested randomly
                    pi_bar_rem = min(Remainder_tests./Remainder_traceable,1); % Probability of getting tested remainder 

                    % Aggregate Testing Probability 
                    % The second part is conditional on not being traced in the first step,
                    % probability of getting test x False negative rate
                    pf.pic_A(t,1)  =  pf.pic_A_tt(t,1) + (1-pi_C_A_tt)*pi_bar_rem*(1-P.pi_F);
                    pf.pic_T(t,1)  =  pf.pic_T_tt(t,1) + (1-pi_C_T_tt)*pi_bar_rem*(1-P.pi_F);

                elseif P.trace == 7  % No testing, only quarantining!
                    pf.pic_A(t,1)  = 0;pf.pic_A_tt(t,1);
                    pf.pic_T(t,1)  = 0;pf.pic_T_tt(t,1);

                    % Quarantine probabilities. Usually, they are set to zero!
                    pf.piq_ST(t,1) = pi_C_ST_tt;
                    pf.piq_T(t,1)  = pi_C_T_tt;
                    pf.piq_A(t,1)  = pi_C_A_tt;
                    pf.piq_U(t,1)  = pi_C_RA_tt;
                elseif P.trace == 8  % Only quarantining after testing
                    pf.pic_A(t,1)  = pf.pic_A_tt(t,1);
                    pf.pic_T(t,1)  = pf.pic_T_tt(t,1);

                    if pf.E_tt(t,1) > pf.Upsilon(t,1)  % Tests are not enough - quarantine everyone that does not get tested
                        ratio =  (pf.E_tt(t,1) - pf.Upsilon(t,1))/pf.E_tt(t,1);  %Calculate the ratio to adjust
                        pf.piq_ST(t,1) = ratio*pi_C_ST_tt;
                        pf.piq_T(t,1)  = ratio*pi_C_T_tt;
                        pf.piq_A(t,1)  = ratio*pi_C_A_tt;
                        pf.piq_U(t,1)  = ratio*pi_C_RA_tt;
                    end
                end


            if P.trace == 2  % Scenario 2
            if t > 1 % This can only apply from the second period onwards

                % Contact Tracing (Section 3.3)
                f_AT = pmf_contacts_withT_infect(pf.c_s(t-1,1),pf.n_s(t-1,1),t-1,P,pf);  % A_{t-1} T_{t-1}
                f_AA = pmf_contacts_withA(pf.c_s(t-1,1),pf.n_s(t-1,1),t-1,P,pf);         % A_{t-1} A_{t-1}
                f_TT = pmf_contacts_withT(pf.c_s(t-1,1),pf.n_s(t-1,1),t-1,P,pf);         % T_{t-1} T_{t-1}
                f_TA = (f_AA.*tau_tilde)/pf.tau_exact(t-1,1);                                  % T_{t-1} A_{t-1} 
                f_ST = f_TT;                                                             % S_t     T_{t-1}
                f_SA = (f_AA.*(1-tau_tilde))/(1-pf.tau_exact(t-1,1));                          % S_t     A_{t-1}
                f_RT = f_ST;                                                             % R_{t-1} T_{t-1}
                f_RA = f_AA;                                                             % R_{t-1} T_{t-1}   
                f_RAT = f_AT;                                                             % RA_{t-1} T_{t-1}
                f_RAA = f_AA;                                                             % RA_{t-1} T_{t-1}               
                % Time Adjustments (see Section 3.3)
                f_TA_adj_temp = f_TA.*(1-(1-(1-P.pii).^P.grid')*pf.pi_bar_tt(t-1,1)*(1-P.pi_F));  
                f_TA = f_TA_adj_temp./(sum(f_TA_adj_temp));                                   
                f_AA_adj_temp = f_AA.*(1-(1-(1-P.pii).^P.grid')*pf.pi_bar_tt(t-1,1)*(1-P.pi_F));  
                f_AA = f_AA_adj_temp./(sum(f_AA_adj_temp));                                                


                % Active Contacts 
                % Type T Links Sampling         
                gG_ST = zeros(P.k_s_max+1,P.k_s_max+1);             
                prob_ST = (1-pf.pic_T(t-1,1));                      % Probability that contact is active
                for i= 0:P.k_s_max
                    gG_ST(:,i+1) = binopdf(P.grid,i,prob_ST);       % Draw from Binomial Distribution
                end   
                gG_TT = gG_ST;                                      % T_{t-1} T_{t-1}; 
                gG_RT = gG_ST;                                      % T_{t-1} T_{t-1}; 
                gG_AT = zeros(P.k_s_max+1,P.k_s_max+1);             % A_{t-1} T_{t-1}

                f_SA_T_L = pf.fk(t-1,:)'.*tau_tilde/pf.tau_exact(t-1,1);  % f_{t-1}^T(k)
                pi_tilde_C_T_L = (1 - (1-P.tau).^(P.grid-1))*f_SA_T_L;  
                pi_tilde_P_0T_L = pi_tilde_C_T_L*pf.pi_bar_tt(t-1,1)*(1-P.pi_F);             
                pi_tilde_P_T_L  = pi_tilde_P_0T_L + (1-pi_tilde_C_T_L)*pf.pic_T_ttl(t-1,1);  
                prob_AT_weight = P.tau/(P.tau+(1-P.tau)*pf.tau_exact(t-1,1));
                prob_AT = 1- (prob_AT_weight*(pi_tilde_P_T_L) + (1-prob_AT_weight)*pf.pic_T(t-1,1));  
                for i= 0:P.k_s_max
                    gG_AT(:,i+1) = binopdf(P.grid,i,prob_AT);       % Draw from Binomial Distribution
                end               
                gG_RAT = gG_AT;   

               %  Type T Links Aggregating - Derive the distribution of underline k (marginalize k!)     
               % For this, we now want to sum up to get the unconditional distribution
               g_ST = gG_ST*f_ST;  
               g_RT = gG_RT*f_RT;  
               g_TT = gG_TT*f_TT;  
               g_AT = gG_AT*f_AT;  
               g_RAT = gG_RAT*f_RAT;  

               % Type A Links Sampling
               gG_tilde_iA = zeros(P.k_s_max+1,P.k_s_max+1);             
               gG_hat_iA = zeros(P.k_s_max+1,P.k_s_max+1);             
               prob_tilde_iA = (1-P.pir)*(1-pf.pic_A(t-1,1));            % Probability that contact is active if not traced
               prob_hat_iA = (1-P.pir-P.pii)*(1-pf.pic_A(t-1,1));      % Probability that contact is active if traced
               for i= 0:P.k_s_max
                   gG_tilde_iA(:,i+1) = binopdf(P.grid,i,        prob_tilde_iA);       % Equation 51
                   gG_hat_iA(:,i+1)   = binopdf(P.grid,max(i-1,0),prob_hat_iA);  
               end    

               % Weights as in Section 3.4
               iota_S = (1-P.pii).^P.grid';  % Equation 53
               iota_T = ((1-P.pii).^P.grid')./((1-P.pii).^P.grid' + (1-(1-P.pii).^P.grid')*(1-pf.pi_bar_tt(t-1,1)*(1-P.pi_F)));  % Equation 54
               iota_A = iota_T;  
               iota_RA = (1-P.pii).^P.grid'; 

               gG_SA = (iota_S)'.*gG_tilde_iA + (1-iota_S)'.*gG_hat_iA;  
               gG_TA = (iota_T)'.*gG_tilde_iA + (1-iota_T)'.*gG_hat_iA;  
               gG_AA = (iota_A)'.*gG_tilde_iA + (1-iota_A)'.*gG_hat_iA;  
               gG_RAA = (iota_RA)'.*gG_tilde_iA + (1-iota_RA)'.*gG_hat_iA;  

                %  Type T Links Aggregating - Derive the distribution of underline k (marginalize k!)     
               % For this, we now want to sum up to get the unconditional distribution
               g_SA = gG_SA*f_SA;  % Equation 44
               g_RA = gG_SA*f_RA;  % Equation 44
               g_TA = gG_TA*f_TA;  % Euqation 44
               g_AA = gG_AA*f_AA;  % Equation 44           
               g_RAA = gG_RAA*f_RAA;  % Equation 44  

               % Tracing Probabilities   % We first derive Equation 57
               g_S = zeros(P.k_s_max+1,1); 
               g_R = zeros(P.k_s_max+1,1);
               g_T = zeros(P.k_s_max+1,1); 
               g_A = zeros(P.k_s_max+1,1); 
               g_RA = zeros(P.k_s_max+1,1); 
               for k = 1:P.k_s_max+1  % Loop over all potential outcomes
                   out_S= [];
                   out_R= [];
                   out_T= [];
                   out_A= [];
                   out_RA= [];
                   % Sum up similarly to the description in the paper for k1,k2,k3
                   for i = 1:k
                       out_t_S = g_SA(i)*g_ST(k+1-i);
                       out_t_R = g_RA(i)*g_RT(k+1-i);
                       out_t_T = g_TA(i)*g_TT(k+1-i);
                       out_t_A = g_AA(i)*g_AT(k+1-i);
                       out_t_RA = g_RAA(i)*g_RAT(k+1-i);
                       out_S = [out_S out_t_S];
                       out_R = [out_R out_t_R];
                       out_T = [out_T out_t_T];
                       out_A = [out_A out_t_A];
                       out_RA = [out_RA out_t_RA];
                   end
                   g_S(k) = sum(out_S);
                   g_R(k) = sum(out_R);
                   g_T(k) = sum(out_T);
                   g_A(k) = sum(out_A);
                   g_RA(k) = sum(out_RA);
               end

               % Final Tracing Probabilities
               pi_C_S_ttl = g_S'*mu_tilde;
               pi_C_R_ttl = g_R'*mu_tilde;
               pi_C_T_ttl = g_T'*mu_tilde;
               pi_C_A_ttl = g_A'*mu_tilde;
               pi_C_RA_ttl = g_RA'*mu_tilde;

               % Exposed Subjects 
               w_A1 = (pf.A(t-1,1)*(1-P.pii-P.pir)*(1-pf.pic_A(t-1,1)))/pf.A(t,1);  % Weight for Type A Links in Group of Asymptomatics

               pf.E_ttl(t,1) =  (1-pi_C_S_tt)*pi_C_S_ttl*pf.S(t,1) + ...
                                (1-pi_C_A_tt)*(w_A1*pi_C_A_ttl+(1-w_A1)*pi_C_T_ttl ) *(1-P.pii)*pf.A(t,1) + ...
                                (1-pi_C_RA_tt)*(pi_C_R_ttl *pf.RA(t-1,1) + pi_C_RA_ttl*P.pir*pf.A(t-1,1));

               % Probability of geting Tested            
               pf.pi_bar_ttl(t,1) = min(max(pf.Upsilon(t,1)-pf.E_tt(t,1),0)/pf.E_ttl(t,1),1);  

               % Probabilities of getting Tested 
               pf.pic_S_ttl(t,1) = pi_C_S_ttl*pf.pi_bar_ttl(t,1)*(1-P.pi_F);
               pf.pic_A_ttl(t,1) = pi_C_A_ttl*pf.pi_bar_ttl(t,1)*(1-P.pi_F);
               pf.pic_T_ttl(t,1) = pi_C_T_ttl*pf.pi_bar_ttl(t,1)*(1-P.pi_F);

               % Probability for an Asymptomatic Infected to Get Tested
               pic_A_hat = pf.pic_A_tt(t,1) + (1-pi_C_A_tt)*pf.pic_A_ttl(t,1);    
               pic_A_tilde = pf.pic_A_tt(t,1) + (1-pi_C_A_tt)*pf.pic_T_ttl(t,1);  
               pf.pic_A(t,1) = w_A1*pic_A_hat + (1-w_A1)*pic_A_tilde;             

               % Probability for a Newly Infected 
                pf.pic_T(t,1)  = pf.pic_T_tt(t,1) + (1-pi_C_T_tt)*pf.pic_S_ttl(t,1);

            end
            end


            elseif P.trace == 11  % Random Testing

                % Contacts
                pf.E_tt(t,1) = pf.S(t,1) + (1-P.pii)*pf.A(t,1) + pf.RA(t,1);
                 % Probability of getting tested conditional on being traced
                pf.pi_bar_tt(t,1) = min(pf.Upsilon(t,1)./pf.E_tt(t,1),1);   

                % Aggregate 
                pf.pic_A(t,1)  = min(pf.Upsilon(t,1)./pf.E_tt(t,1),1)*(1-P.pi_F);
                pf.pic_T(t,1)  = min(pf.Upsilon(t,1)./pf.E_tt(t,1),1)*(1-P.pi_F);


            end

            % Evolvement of Groups
            pf.S(t+1,1) = (1-pf.piq_ST(t,1)) * (pf.S(t,1) - pf.T(t,1)) + P.piq*pf.QS(t,1);   % Susceptible  
            pf.A(t+1,1) = (1-P.pii-P.pir)*(1-pf.pic_A(t,1)-pf.piq_A(t,1))*pf.A(t,1) + ...
                            (1-pf.pic_T(t,1)-pf.piq_T(t,1))*pf.T(t,1) + P.piq*(1-P.pii-P.pir)* pf.QA(t,1);
            pf.C(t+1,1) = (1-P.pii-P.pir)*pf.C(t,1) + (1-P.pii-P.pir)*pf.pic_A(t,1)*pf.A(t,1) ...
                           + pf.pic_T(t,1)*pf.T(t,1);                  
            pf.I(t+1,1) = pf.I(t,1) + P.pii*(pf.A(t,1)+ pf.C(t,1)+pf.QA(t,1)) - (pf.pid_t(t,1) + P.pir)*pf.I(t,1);
            pf.RA(t+1,1) = (1-pf.piq_U(t,1))*pf.RA(t,1) + P.pir*(pf.A(t,1)) + P.piq*pf.QU(t,1) + P.piq*P.pir*pf.QA(t,1);
            pf.RI(t+1,1) = pf.RI(t,1) + P.pir*(pf.C(t,1)) +  P.pir*pf.I(t,1);
            pf.R(t+1,1) = pf.RI(t+1,1) + pf.RA(t+1,1);
            pf.D(t+1,1) = pf.D(t,1) + pf.pid_t(t,1)*pf.I(t,1);

            % Quarantine
            pf.QS(t+1,1) = (1-P.piq)*pf.QS(t,1) +  pf.piq_ST(t,1)* (pf.S(t,1) - pf.T(t,1));
            pf.QA(t+1,1) = (1-P.piq)*(1-P.pii-P.pir)*pf.QA(t,1) + pf.piq_T(t,1)*pf.T(t,1) + (1-P.pii-P.pir)*pf.piq_A(t,1)*pf.A(t,1) ;                          
            pf.QU(t+1,1) = (1-P.piq)*pf.QU(t,1) + pf.piq_U(t,1)*pf.RA(t,1) + (1-P.piq)*P.pir*pf.QA(t,1) ;
            pf.Q(t+1,1) = pf.QS(t+1,1) + pf.QA(t+1,1) + pf.QU(t+1,1);

            % Level of Population        
            pf.Pop(t+1,1) = pf.S(t+1,1) + pf.I(t+1,1) + pf.R(t+1,1) + pf.C(t+1,1) + pf.A(t+1,1) + pf.Q(t+1,1); % - pf.D(t+1,1)   
            end
        end


        % Iterate backwards again to calculate utility
        for t = T:-1:1


            % Utility of infected function
            pf.Util_i(t,1) = pf.u_i(t,1) + P.beta.*( (1-pf.pid_t(t,1) - P.pir)*pf.Util_i(t+1,1) + P.pir*pf.Util_r(t+1,1) );


            % Utility of checked infected
            pf.Util_c(t,1) = pf.u_c(t,1) + P.beta.*( P.pii*pf.Util_i(t+1,1)  + P.pir*pf.Util_r(t+1,1) ...
                                                      + (1-P.pii-P.pir)*pf.Util_c(t+1,1)          ) ;

            % Utility of susceptibles
            pf.Util_s(t,1) = pf.u_s(t,1) + P.beta.*( (1-pf.tau(t,1)).* (  (1-pf.piq_ST(t,1))*pf.Util_s(t+1,1) +   pf.piq_ST(t,1)*pf.Util_qs(t+1,1)  )...
                + pf.tau(t,1).*(  (1-pf.pic_T(t,1)-pf.piq_T(t,1) )* pf.Util_a(t+1,1) + pf.pic_T(t,1)*pf.Util_c(t+1,1) + pf.piq_T(t,1)*pf.Util_qa(t+1,1) ) );

             % Utility of asymptotic infected
             pf.Util_a(t,1) = pf.u_s(t,1) + P.beta.*( P.pii*pf.Util_i(t+1,1)  + P.pir*pf.Util_ur(t+1,1) ...
                             + (1-P.pii-P.pir)*(pf.pic_A(t,1)*pf.Util_c(t+1,1) + pf.piq_A(t,1)*pf.Util_qa(t+1,1) + (1-pf.pic_A(t,1)-pf.piq_A(t,1))*pf.Util_a(t+1,1) )   );

            % Utility of undetected recovered  
            pf.Util_ur(t,1) =  pf.u_s(t,1) + P.beta.*((1-pf.piq_U(t,1))*pf.Util_ur(t+1,1) + pf.piq_U(t,1)*pf.Util_qu(t+1,1) );

            % Utility of quarantined agents
            % Susceptible
            pf.Util_qs(t,1) = pf.u_q(t,1) + P.beta * (P.piq*pf.Util_s(t+1,1) + (1-P.piq)*pf.Util_qs(t+1,1));
            % Asymptomatic Infected
            pf.Util_qa(t,1) = pf.u_q(t,1) + P.beta * (P.pii*pf.Util_i(t+1,1) + P.piq* (P.pir*pf.Util_ur(t+1,1) + (1-P.pii-P.pir)*pf.Util_a(t,1)  ) + ...
                                                       + (1-P.piq)*(P.pir*pf.Util_qu(t+1,1)  + (1-P.pii-P.pir)*pf.Util_qa(t+1,1) ) );
            % Unobserved recovered
            pf.Util_qu(t,1) = pf.u_q(t,1) + P.beta * (P.piq*pf.Util_ur(t+1,1) + (1-P.piq)*pf.Util_qu(t+1,1) );
        end

        % Proceed forwards to calculate some remaining parameters

        for t = 1:T
            % changes with it
            pf.lambdatau_s(t,1) = P.beta*((  (1-pf.pic_T(t,1)-pf.piq_T(t,1))* pf.Util_a(t+1) + pf.pic_T(t,1)*pf.Util_c(t+1) + pf.piq_T(t,1)*pf.Util_qa(t+1)) ...
            - ( (1-pf.piq_ST(t,1))*pf.Util_s(t+1,1) + pf.piq_ST(t,1)*pf.Util_qs(t+1,1) ) );
            pf.lambdab_s(t,1)  = (1./(1+pf.lambda_c(t,1)))*(pf.c_s(t,1).^(-1) + pf.lambdatau_s(t,1)*pf.pi_s1(t,1)*( pf.A(t,1)*pf.c_s(t,1)./pf.Cons_Meet(t,1) ) );
        end

        % System of Equations to be solved each period
        % Final Labor

        for t= 1:T

            start  = [pf.n_r(t,1) pf.n_i(t,1) pf.n_s(t,1) pf.n_c(t,1) pf.n_q(t,1) ]';
            argzero = csolve('eqm_SIRAC_microfounded_BR_Exposure',start,[],1e-8,20,t,pf,P);

            pf_up_n_r(t,1) = argzero(1); 
            pf_up_n_i(t,1) = argzero(2);
            pf_up_n_s(t,1) = argzero(3);
            pf_up_n_c(t,1) = argzero(4);
            pf_up_n_q(t,1) = argzero(5);

        end

        pf.n_r = P.up_w*pf.n_r + (1-P.up_w)*pf_up_n_r;
        pf.n_i = P.up_w*pf.n_i + (1-P.up_w)*pf_up_n_i;
        pf.n_s = P.up_w*pf.n_s + (1-P.up_w)*pf_up_n_s;
        pf.n_c = P.up_w*pf.n_c + (1-P.up_w)*pf_up_n_c;
        pf.n_q = P.up_w*pf.n_q + (1-P.up_w)*pf_up_n_q;

        dist_nr = max(abs(pf.n_r(:) -pf_up_n_r(:)));
        dist_ni = max(abs(pf.n_i(:) -pf_up_n_i(:)));
        dist_ns = max(abs(pf.n_s(:) -pf_up_n_s(:)));
        dist_nc = max(abs(pf.n_c(:) -pf_up_n_c(:)));
        dist_nq = max(abs(pf.n_q(:) -pf_up_n_q(:)));

        dist_total = max([dist_nr dist_ni dist_ns dist_nc dist_nq]);

        if dist_total < P.tol 
            converged = 1;
        end

        %disp(['Iteration: ' num2str(j) ' Distance: ' num2str(dist_total)])
        j = j+1;

    end

 end

% % Utility
pf.U = pf.S.*pf.Util_s(1:end-1) + pf.I.*pf.Util_i(1:end-1) + pf.RA.*pf.Util_r(1:end-1) + pf.RI.*pf.Util_r(1:end-1) + pf.A.*pf.Util_a(1:end-1) + pf.C.*pf.Util_c(1:end-1)  + pf.QS.*pf.Util_qs(1:end-1)+ pf.QA.*pf.Util_qa(1:end-1)+ pf.QU.*pf.Util_qu(1:end-1);


pf.R_0 = (1-pf.pic_T(:)-pf.piq_T(:)).*(pf.T(:)./pf.A)./(P.pir+P.pii + (1-P.pir-P.pii)*(pf.pic_A(:)+ pf.piq_A(:))); % ./(P.pir+P.pid+P.pii)

