% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This code helps to solve the optimal policy for the economy without 
% tracing
% =========================================================================

function [U0]=getU_err(muc_guess,pf_n_s_guess,pf_n_i_guess,pf_n_r_guess,P,T)


%muc=zeros(T,1);
muc(1:numel(muc_guess))=muc_guess/1;

persistent guess_n_s guess_n_i guess_n_r n_s_vec n_i_vec n_r_vec
if isempty(guess_n_s)==1
    guess_n_s=pf_n_s_guess;
    guess_n_i=pf_n_i_guess;
    guess_n_r=pf_n_r_guess;
    try
    [pf_U,guess_n_s,guess_n_i,guess_n_r] = eqm_Exposure_get_err(guess_n_s,guess_n_i,guess_n_r,muc,P,T);
    catch
    end
else
    guess_n_s=n_s_vec;
    guess_n_i=n_i_vec;
    guess_n_r=n_r_vec;
end



try
 
  opts_fsolve_fmincon=optimoptions('fsolve','Display','off','TolFun',1e-9,'MaxFunctionEvaluations',10000);
   guess = [guess_n_s;guess_n_i;guess_n_r];  
   [n_vec,fval,exitflag]=fsolve(@eqm_Exposure_get_err_1d,guess,opts_fsolve_fmincon,muc,P,T);
                                                              
    if exitflag~=1
        error('Fsolve could not solve the model');
    end
 
[err,pf_U,n_s_vec,n_i_vec,n_r_vec] =eqm_Exposure_get_err_1d(n_vec,muc,P,T);
  U0 = -100*pf_U(1); 

catch
    try
        %2
        disp(num2str(2))
        [pf_U,guess_n_s,guess_n_i,guess_n_r] = eqm_Exposure_get_err(guess_n_s,guess_n_i,guess_n_r,muc,P,T);
        opts_fsolve_fmincon=optimoptions('fsolve','Display','off','TolFun',1e-9);
        guess = [guess_n_s;guess_n_i;guess_n_r];
        [n_vec,fval,exitflag]=fsolve(@eqm_Exposure_get_err_1d,guess,opts_fsolve_fmincon,muc,P,T);
                                                                
        if exitflag~=1
            error('Fsolve could not solve the model');
        end
        
        [err,pf_U,n_s_vec,n_i_vec,n_r_vec] =eqm_Exposure_get_err_1d(n_vec,muc,P,T);
        U0 = -100*pf_U(1); 
        
    catch
        
        try
            %3
             disp(num2str(3))
            % Try with alternative guess
            guess_n_s=pf_n_s_guess;
            guess_n_i=pf_n_i_guess;
            guess_n_r=pf_n_r_guess;
            
            [pf_U,guess_n_s,guess_n_i,guess_n_r] = eqm_Exposure_get_err(guess_n_s,guess_n_i,guess_n_r,muc,P,T);
            opts_fsolve_fmincon=optimoptions('fsolve','Display','off','TolFun',1e-9);
            guess = [guess_n_s;guess_n_i;guess_n_r];
            [n_vec,fval,exitflag]=fsolve(@eqm_Exposure_get_err_1d,guess,opts_fsolve_fmincon,muc,P,T);

            if exitflag~=1
                error('Fsolve could not solve the model');
            end

            [err,pf_U,n_s_vec,n_i_vec,n_r_vec] =eqm_Exposure_get_err_1d(n_vec,muc,P,T);
            U0 = -100*pf_U(1); 
           
        catch
            %4
            U0 = 100*1e10;
            guess_n_s = [];
            disp('Could not solve the model')
            disp(num2str(4))
        end

    end
end

end