% =========================================================================
% Code for "Pandemic Recessions and Contact Tracing"
% by Leonardo Melosi and Matthias Rottner
% Journal of the European Economic Association, forthcoming
% This file provides calibration and some options
% =========================================================================

addpath Simulations
opts.saveFolder  = 'Figures/';
opts.saveSimulations  = 'Simulations/'; 
% Horizon of the model
T = 250;
    
% Tracing Technology; 1 = current week tracing; 2 = current and previous;
% 7= Only exposed current week; 8 = Exposed only "leftovers" current week
% after tests
% week tracing
P.trace = 1;  11; 2; 11;  7; 8;
P.Choice_Ups = 2; % 1 10% of Population in tests, 2 limited tests
P.Choice_lock = 1; % 1 no lockdown, else lockdown 

P.Ups_flatrate = 0.1;  % 10% available entire horizon
% Test Availability
Pop_US = 329314; % in thousands %https://fred.stlouisfed.org/series/POPTHM
Test_w11 = 30; % in thousands; daily test week 11 2020 approx
Test_w40 = 1000; % ; daily test week 40 2020 approx see https://coronavirus.jhu.edu/testing/individual-states
P.Ups_start_w11 = Test_w11*7/Pop_US;  
P.Ups_end_w40 = Test_w40*7/Pop_US;  

P.lock_start= 1;
P.lock_end = 52;
P.lockdown = 0.2;

% Update policy functions
P.up_w = 0.975;

P.tau = 0.05;% Probability to get infected from a contact
P.k_s_max = 10;  % Maximum amount of contacts that we keep track of!
P.grid = 0:P.k_s_max; % Grid to calculate the probability density function 

% Calibration
P.tol     = 1e-1;
P.beta    = 0.96^(1/52);  % Discount factor
P.alpha   = 0;
P.phi_c   = 1;  % Reduction in Work checked
P.phi_q   = 1;  % Reduction in Worked quarantined
P.alpha_c = 0;  % Degree that quarantine affects already quarantined once
P.mu_cQ   = 1;  % Quarantine for agents
P.phi     = 0.8; % % Reduction in Work
P.pir     = 7/18  ;   % Probaiblity of Recovery
P.piq     = 0.5;      % Probability of leaving quarantine
Share_Symptoms = 0.5;
P.pii     = P.pir*Share_Symptoms/(1-Share_Symptoms);  % Probability from asymptotic to infected
IFR = 0.003;
P.pid     = 1/(Share_Symptoms)*IFR;  % Corresponds to a 0.5% probability to die
P.pic     = 0.0;   % Probability of checked
P.epsilon = 0.0001; % Initial ill   
P.start_eps = 1;
P.start_nguess = 1;
% We distribute the weights based on a simulation starting with 0.00001 
P.eps_A =  0.5;0.4952;  % Share of asymptomatic
P.eps_I =  0.5;0.1669;
P.eps_RA = 0;0.2484;
P.eps_RI = 0;0.0896;


P.kappa   = 0;  % Med Prep: 0,12.5
P.pi_F     = 0;  % False negatives
P.eta     = 0.5;  % Frisch elasticity
P.RplusD_target = 0.88; % Some target regarding herd immunity

%Calibration targets for hours and income
n_target=28;         %Weekly hours
inc_target=58000/52; %weekly income
P.N    = n_target;     
P.A =  39.835165;  % Productivity
P.Y = P.A*P.N;
P.C = P.Y;
P.w = P.A;
P.lambda = 1/P.C;
P.theta = P.w*P.lambda*P.N^(P.eta/(P.eta-1));

steadystate

% Set starting values
P.varphi = 10;  % 
P.varphi_c = 1*P.varphi/P.C; 
P.varphi_n = 1*P.varphi/P.N; 

% Calibration strategy
P.chi = 1; 

